/*
Copyright 2020 The SuperEdge Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package conf

import (
	"github.com/BurntSushi/toml"
	"github.com/superedge/superedge/pkg/tunnel/util"
)

var TunnelConf *Tunnel

type Tunnel struct {
	TunnlMode *TunnelMode `toml:"mode"`
}

type TunnelMode struct {
	Cloud *TunnelCloud `toml:"cloud"`
	EDGE  *TunnelEdge  `toml:"edge"`
}

type TunnelCloud struct {
	Https  *HttpsServer      `toml:"https"`
	Stream *StreamCloud      `toml:"stream"`
	Tcp    map[string]string `toml:"tcp"`
	Egress *EgressServer     `toml:"egress"`
}

type HttpsServer struct {
	Cert string            `toml:"cert"`
	Key  string            `toml:"key"`
	Addr map[string]string `toml:"addr"`
}

type StreamCloud struct {
	Server *StreamServer `toml:"server"`
	Dns    *Dns          `toml:"dns"`
}

type StreamServer struct {
	TokenFile    string `toml:"tokenfile"`
	Key          string `toml:"key"`
	Cert         string `toml:"cert"`
	GrpcPort     int    `toml:"grpcport"`
	LogPort      int    `toml:"logport"`
	MetricsPort  int    `toml:"metricsport"`
	ChannelzAddr string `toml:"channelzaddr"`
}

type EgressServer struct {
	ServerCert string `toml:"servercert"`
	ServerKey  string `toml:"serverkey"`
	KubeConfig string `toml:"kubeconfig"`
	EgressPort string `toml:"egressport"`
}

type Dns struct {
	Configmap string `toml:"configmap"`
	Hosts     string `toml:"hosts"`
	Service   string `toml:"service"`
	Debug     bool   `toml:"debug"`
}

type TunnelEdge struct {
	Https      *HttpsClient `toml:"https"`
	StreamEdge StreamEdge   `toml:"stream"`
}

type HttpsClient struct {
	Cert string `toml:"cert"`
	Key  string `toml:"key"`
}

type StreamEdge struct {
	Client *StreamClient `toml:"client"`
}

type StreamClient struct {
	Token        string `toml:"token"`
	Cert         string `toml:"cert"`
	Dns          string `toml:"dns"`
	ServerName   string `toml:"servername"`
	LogPort      int    `toml:"logport"`
	ChannelzAddr string `toml:"channelzaddr"`
}

type Config struct {
	Mode map[string]map[string]map[string]interface{} `toml:"mode"`
}

func InitConf(mode, path string) error {
	if mode == util.CLOUD {
		TunnelConf = &Tunnel{
			TunnlMode: &TunnelMode{
				Cloud: &TunnelCloud{},
			},
		}
	} else {
		TunnelConf = &Tunnel{
			TunnlMode: &TunnelMode{
				EDGE: &TunnelEdge{},
			},
		}
	}
	_, err := toml.DecodeFile(path, TunnelConf)
	if err != nil {
		return err
	}
	return nil
}
